@@ -30,28 +30,17 @@ from driftpy.math.positions import is_available, is_spot_position_available
 
 DEFAULT_USER_NAME = "Main Account"
 
-
 class ClearingHouse:
-    """This class is the main way to interact with Drift Protocol.
-
-    It allows you to subscribe to the various accounts where the Market's state is
-    stored, as well as: opening positions, liquidating, settling funding, depositing &
-    withdrawing, and more.
-
-    The default way to construct a ClearingHouse instance is using the
-    [create][driftpy.clearing_house.ClearingHouse.create] method.
+    """This class is the main way to interact with Drift Protocol including 
+    depositing, opening new positions, closing positions, placing orders, etc.
     """
 
     def __init__(self, program: Program, authority: Keypair = None):
-        """Initialize the ClearingHouse object.
-
-        Note: you probably want to use
-        [create][driftpy.clearing_house.ClearingHouse.create]
-        instead of this method.
+        """Initializes the clearing house object -- likely want to use the .from_config method instead of this one
 
         Args:
-            program: The AnchorPy program object.
-            pdas: The required PDAs for the ClearingHouse object.
+            program (Program): Drift anchor program (see from_config on how to initialize it)
+            authority (Keypair, optional): Authority of all txs - if None will default to the Anchor Provider.Wallet Keypair.
         """
         self.program = program
         self.program_id = program.program_id
@@ -65,10 +54,20 @@ class ClearingHouse:
         self.signers = [self.signer]
         self.usdc_ata = None
         self.spot_market_atas = {}
-        self.subaccounts = None
+        self.subaccounts = [0]
 
     @staticmethod
     def from_config(config: Config, provider: Provider, authority: Keypair = None):
+        """Initializes the clearing house object from a Config 
+
+        Args:
+            config (Config): the config to initialize form
+            provider (Provider): anchor provider
+            authority (Keypair, optional):  _description_. Defaults to None.
+
+        Returns:
+            ClearingHouse: the clearing house object
+        """
         # read the idl
         file = Path(str(driftpy.__path__[0]) + "/idl/drift.json")
         with file.open() as f:
@@ -118,6 +117,14 @@ class ClearingHouse:
         return await self.program.provider.send(tx, signers=signers)
 
     async def intialize_user(self, user_id: int = 0):
+        """intializes a drift user 
+
+        Args:
+            user_id (int, optional): subaccount id to initialize. Defaults to 0.
+
+        Returns:
+            str: tx signature
+        """
         ixs = []
         if user_id == 0:
             ixs.append(self.get_initialize_user_stats())
@@ -188,7 +195,7 @@ class ClearingHouse:
         writable_market_index: int = None,
         writable_spot_market_index: int = None,
         readable_spot_market_index: int = None,
-        user_id=0,
+        user_id=[0],
         include_oracles: bool = True,
         include_spot_markets: bool = True,
         authority: Optional[Union[PublicKey, Sequence[PublicKey]]] = None,
@@ -198,9 +205,13 @@ class ClearingHouse:
         elif isinstance(authority, PublicKey):
             authority = [authority]
 
+        if isinstance(user_id, int):
+            user_id = [user_id]
+        assert len(user_id) == len(authority)
+
         accounts = []
-        for pk in authority:
-            user_account = await get_user_account(self.program, pk, user_id)
+        for pk, id in zip(authority, user_id):
+            user_account = await get_user_account(self.program, pk, id)
             accounts.append(user_account)
 
         oracle_map = {}
@@ -279,6 +290,18 @@ class ClearingHouse:
         reduce_only: bool = False,
         user_id: int = 0,
     ):
+        """withdraws from drift protocol (can also allow borrowing)
+
+        Args:
+            amount (int): amount to withdraw
+            spot_market_index (int): 
+            user_token_account (PublicKey): ata of the account to withdraw to
+            reduce_only (bool, optional): if True will only withdraw existing funds else if False will allow taking out borrows. Defaults to False.
+            user_id (int, optional): subaccount. Defaults to 0.
+
+        Returns:
+            str: tx sig
+        """
         return await self.send_ixs(
             [
                 await self.get_withdraw_collateral_ix(
@@ -333,6 +356,19 @@ class ClearingHouse:
         reduce_only=False,
         user_initialized=True,
     ):
+        """deposits collateral into protocol
+
+        Args:
+            amount (int): amount to deposit
+            spot_market_index (int): 
+            user_token_account (PublicKey): 
+            user_id (int, optional): subaccount to deposit into. Defaults to 0.
+            reduce_only (bool, optional): paying back borrow vs depositing new assets. Defaults to False.
+            user_initialized (bool, optional): if need to initialize user account too set this to False. Defaults to True.
+
+        Returns:
+            str: sig
+        """
         return await self.send_ixs(
             [
                 await self.get_deposit_collateral_ix(
@@ -390,6 +426,16 @@ class ClearingHouse:
         )
 
     async def add_liquidity(self, amount: int, market_index: int, user_id: int = 0):
+        """mint LP tokens and add liquidity to the DAMM
+
+        Args:
+            amount (int): amount of lp tokens to mint
+            market_index (int): market you want to lp in
+            user_id (int, optional): subaccount id. Defaults to 0.
+
+        Returns:
+            str: tx sig
+        """
         return await self.send_ixs(
             [await self.get_add_liquidity_ix(amount, market_index, user_id)]
         )
@@ -418,6 +464,16 @@ class ClearingHouse:
         )
 
     async def remove_liquidity(self, amount: int, market_index: int, user_id: int = 0):
+        """burns LP tokens and removes liquidity to the DAMM
+
+        Args:
+            amount (int): amount of lp tokens to burn
+            market_index (int): 
+            user_id (int, optional): subaccount id. Defaults to 0.
+
+        Returns:
+            str: tx sig
+        """
         return await self.send_ixs(
             [await self.get_remove_liquidity_ix(amount, market_index, user_id)]
         )
@@ -444,6 +500,14 @@ class ClearingHouse:
         )
 
     async def cancel_orders(self, user_id: int = 0):
+        """cancel all existing orders on the book
+
+        Args:
+            user_id (int, optional): subaccount id. Defaults to 0.
+
+        Returns:
+            str: tx sig
+        """
         return await self.send_ixs(await self.get_cancel_orders_ix(user_id))
 
     async def get_cancel_orders_ix(self, user_id: int = 0):
@@ -468,6 +532,15 @@ class ClearingHouse:
         order_id: Optional[int] = None,
         user_id: int = 0,
     ):
+        """cancel specific order (if order_id=None will be most recent order)
+
+        Args:
+            order_id (Optional[int], optional): Defaults to None.
+            user_id (int, optional): subaccount id which contains order. Defaults to 0.
+
+        Returns:
+            str: tx sig
+        """
         return await self.send_ixs(
             await self.get_cancel_order_ix(order_id, user_id),
         )
@@ -754,9 +827,11 @@ class ClearingHouse:
     async def liquidate_spot(
         self,
         user_authority: PublicKey,
-        asset_market_index: int,
-        liability_market_index: int,
+        asset_market_index: int, 
+        liability_market_index: int, 
         max_liability_transfer: int,
+        user_subaccount_id: int = 0,
+        liq_subaccount_id: int = 0,
     ):
         return await self.send_ixs(
             [
@@ -765,6 +840,8 @@ class ClearingHouse:
                     asset_market_index,
                     liability_market_index,
                     max_liability_transfer,
+                    user_subaccount_id,
+                    liq_subaccount_id,
                 )
             ]
         )
@@ -772,23 +849,26 @@ class ClearingHouse:
     async def get_liquidate_spot_ix(
         self,
         user_authority: PublicKey,
-        asset_market_index: int,
-        liability_market_index: int,
-        max_liability_transfer: int,
+        asset_market_index: int, 
+        liability_market_index: int, 
+        max_liability_transfer: int, 
         limit_price: int = None,
+        user_subaccount_id: int = 0,
+        liq_subaccount_id: int = 0,
     ):
-        user_pk = get_user_account_public_key(self.program_id, user_authority)
+        user_pk = get_user_account_public_key(self.program_id, user_authority, user_id=user_subaccount_id)
         user_stats_pk = get_user_stats_account_public_key(
             self.program_id,
             user_authority,
         )
 
-        liq_pk = self.get_user_account_public_key()
+        liq_pk = self.get_user_account_public_key(liq_subaccount_id)
         liq_stats_pk = self.get_user_stats_public_key()
 
         remaining_accounts = await self.get_remaining_accounts(
             writable_spot_market_index=[liability_market_index, asset_market_index],
             authority=[user_authority, self.authority],
+            user_id=[user_subaccount_id, liq_subaccount_id],
         )
 
         return self.program.instruction["liquidate_spot"](
@@ -815,6 +895,8 @@ class ClearingHouse:
         market_index: int,
         max_base_asset_amount: int,
         limit_price: Optional[int] = None,
+        user_subaccount_id: int = 0,
+        liq_subaccount_id: int = 0,
     ):
         return await self.send_ixs(
             [
@@ -823,6 +905,8 @@ class ClearingHouse:
                     market_index,
                     max_base_asset_amount,
                     limit_price,
+                    user_subaccount_id,
+                    liq_subaccount_id,
                 )
             ]
         )
@@ -833,19 +917,22 @@ class ClearingHouse:
         market_index: int,
         max_base_asset_amount: int,
         limit_price: Optional[int] = None,
+        user_subaccount_id: int = 0,
+        liq_subaccount_id: int = 0,
     ):
-        user_pk = get_user_account_public_key(self.program_id, user_authority)
+        user_pk = get_user_account_public_key(self.program_id, user_authority, user_subaccount_id)
         user_stats_pk = get_user_stats_account_public_key(
             self.program_id,
             user_authority,
         )
 
-        liq_pk = self.get_user_account_public_key()
+        liq_pk = self.get_user_account_public_key(liq_subaccount_id)
         liq_stats_pk = self.get_user_stats_public_key()
 
         remaining_accounts = await self.get_remaining_accounts(
-            writable_market_index=market_index,
+            writable_market_index=market_index, 
             authority=[user_authority, self.authority],
+            user_id=[user_subaccount_id, liq_subaccount_id]
         )
 
         return self.program.instruction["liquidate_perp"](
@@ -865,6 +952,26 @@ class ClearingHouse:
             ),
         )
 
+    async def liquidate_perp_pnl_for_deposit(
+        self, 
+        user_authority: PublicKey,
+        perp_market_index: int,
+        spot_market_index: int,
+        max_pnl_transfer: int,
+        user_subaccount_id: int = 0,
+        liq_subaccount_id: int = 0,
+    ):
+        return await self.send_ixs(
+            self.get_liquidate_perp_pnl_for_deposit_ix(
+                user_authority,
+                perp_market_index,
+                spot_market_index,
+                max_pnl_transfer,
+                user_subaccount_id,
+                liq_subaccount_id,
+            )
+        )
+
     async def get_liquidate_perp_pnl_for_deposit_ix(
         self,
         user_authority: PublicKey,
@@ -872,20 +979,23 @@ class ClearingHouse:
         spot_market_index: int,
         max_pnl_transfer: int,
         limit_price: int = None,
+        user_subaccount_id: int = 0,
+        liq_subaccount_id: int = 0,
     ):
-        user_pk = get_user_account_public_key(self.program_id, user_authority)
+        user_pk = get_user_account_public_key(self.program_id, user_authority, user_subaccount_id)
         user_stats_pk = get_user_stats_account_public_key(
             self.program_id,
             user_authority,
         )
 
-        liq_pk = self.get_user_account_public_key()
+        liq_pk = self.get_user_account_public_key(liq_subaccount_id)
         liq_stats_pk = self.get_user_stats_public_key()
 
         remaining_accounts = await self.get_remaining_accounts(
             writable_market_index=perp_market_index,
             writable_spot_market_index=spot_market_index,
-            authority=user_authority,
+            authority=[user_authority, self.authority],
+            user_id=[user_subaccount_id, liq_subaccount_id],
         )
 
         result = self.program.instruction["liquidate_perp_pnl_for_deposit"](
@@ -907,21 +1017,6 @@ class ClearingHouse:
         )
         return result
 
-    async def liquidate_perp_pnl_for_deposit(
-        self,
-        user_authority: PublicKey,
-        perp_market_index: int,
-        spot_market_index: int,
-        max_pnl_transfer: int,
-    ):
-        return await self.send_ixs(
-            self.get_liquidate_perp_pnl_for_deposit_ix(
-                user_authority,
-                perp_market_index,
-                spot_market_index,
-                max_pnl_transfer,
-            )
-        )
 
     async def settle_pnl(
         self,
@@ -970,12 +1065,16 @@ class ClearingHouse:
         self,
         user_authority: PublicKey,
         spot_market_index: int,
+        user_subaccount_id: int = 0,
+        liq_subaccount_id: int = 0,
     ):
         return await self.send_ixs(
             [
                 await self.get_resolve_spot_bankruptcy_ix(
                     user_authority,
                     spot_market_index,
+                    user_subaccount_id,
+                    liq_subaccount_id,
                 )
             ]
         )
@@ -984,22 +1083,30 @@ class ClearingHouse:
         self,
         user_authority: PublicKey,
         spot_market_index: int,
+        user_subaccount_id: int = 0,
+        liq_subaccount_id: int = 0,
     ):
-        user_pk = get_user_account_public_key(self.program_id, user_authority)
+        user_pk = get_user_account_public_key(self.program_id, user_authority, user_subaccount_id)
         user_stats_pk = get_user_stats_account_public_key(
             self.program_id,
             user_authority,
         )
 
-        liq_pk = self.get_user_account_public_key()
+        liq_pk = self.get_user_account_public_key(liq_subaccount_id)
         liq_stats_pk = self.get_user_stats_public_key()
 
         remaining_accounts = await self.get_remaining_accounts(
             writable_spot_market_index=spot_market_index,
             authority=[user_authority, self.authority],
+            user_id=[user_subaccount_id, liq_subaccount_id]
         )
 
-        spot_market = await get_spot_market_account(self.program, spot_market_index)
+        if_vault = get_insurance_fund_vault_public_key(
+            self.program_id, spot_market_index
+        )
+        spot_vault = get_spot_market_vault_public_key(
+            self.program_id, spot_market_index
+        )
         ch_signer = get_clearing_house_signer_public_key(self.program_id)
 
         return self.program.instruction["resolve_spot_bankruptcy"](
@@ -1012,8 +1119,8 @@ class ClearingHouse:
                     "user_stats": user_stats_pk,
                     "liquidator": liq_pk,
                     "liquidator_stats": liq_stats_pk,
-                    "spot_market_vault": spot_market.vault,
-                    "insurance_fund_vault": spot_market.insurance_fund.vault,
+                    "spot_market_vault": spot_vault,
+                    "insurance_fund_vault": if_vault,
                     "drift_signer": ch_signer,
                     "token_program": TOKEN_PROGRAM_ID,
                 },
@@ -1025,12 +1132,16 @@ class ClearingHouse:
         self,
         user_authority: PublicKey,
         market_index: int,
+        user_subaccount_id: int = 0,
+        liq_subaccount_id: int = 0,
     ):
         return await self.send_ixs(
             [
                 await self.get_resolve_perp_bankruptcy_ix(
                     user_authority,
                     market_index,
+                    user_subaccount_id,
+                    liq_subaccount_id,
                 )
             ]
         )
@@ -1039,24 +1150,30 @@ class ClearingHouse:
         self,
         user_authority: PublicKey,
         market_index: int,
+        user_subaccount_id: int = 0,
+        liq_subaccount_id: int = 0,
     ):
-        user_pk = get_user_account_public_key(self.program_id, user_authority)
+        user_pk = get_user_account_public_key(self.program_id, user_authority, user_subaccount_id)
         user_stats_pk = get_user_stats_account_public_key(
             self.program_id,
             user_authority,
         )
 
-        liq_pk = self.get_user_account_public_key()
+        liq_pk = self.get_user_account_public_key(liq_subaccount_id)
         liq_stats_pk = self.get_user_stats_public_key()
 
         remaining_accounts = await self.get_remaining_accounts(
             writable_market_index=market_index,
             writable_spot_market_index=QUOTE_ASSET_BANK_INDEX,
             authority=[user_authority, self.authority],
+            user_id=[user_subaccount_id, liq_subaccount_id]
         )
 
-        spot_market = await get_spot_market_account(
-            self.program, QUOTE_ASSET_BANK_INDEX
+        if_vault = get_insurance_fund_vault_public_key(
+            self.program_id, market_index
+        )
+        spot_vault = get_spot_market_vault_public_key(
+            self.program_id, market_index
         )
         ch_signer = get_clearing_house_signer_public_key(self.program_id)
 
@@ -1071,8 +1188,8 @@ class ClearingHouse:
                     "user_stats": user_stats_pk,
                     "liquidator": liq_pk,
                     "liquidator_stats": liq_stats_pk,
-                    "spot_market_vault": spot_market.vault,
-                    "insurance_fund_vault": spot_market.insurance_fund.vault,
+                    "spot_market_vault": spot_vault,
+                    "insurance_fund_vault": if_vault,
                     "drift_signer": ch_signer,
                     "token_program": TOKEN_PROGRAM_ID,
                 },
@@ -1086,6 +1203,7 @@ class ClearingHouse:
     ):
         return await self.send_ixs(
             [
+                self.get_increase_compute_ix(),
                 await self.get_settle_expired_market_ix(
                     market_index,
                 )
@@ -1232,8 +1350,9 @@ class ClearingHouse:
         )
 
         assert (
-            self.usdc_ata is not None
-        ), "please set self.usdc_ata as your usdc ata pubkey before this ix"
+            self.spot_market_atas[spot_market_index] is not None
+        ), "please set self.spot_market_atas[spot_market_index] as your spot ata pubkey before this ix"
+
         return self.program.instruction["add_insurance_fund_stake"](
             spot_market_index,
             amount,
@@ -1341,3 +1460,27 @@ class ClearingHouse:
                 remaining_accounts=remaining_accounts,
             ),
         )
+
+    async def settle_revenue_to_insurance_fund(
+        self, 
+        spot_market_index: int
+    ):
+        return await self.program.rpc["settle_revenue_to_insurance_fund"](
+            spot_market_index,
+            ctx=Context(
+                accounts={
+                    "state": get_state_public_key(self.program_id),
+                    "spot_market": get_spot_market_public_key(
+                        self.program_id, spot_market_index
+                    ),
+                    "spot_market_vault": get_spot_market_vault_public_key(
+                        self.program_id, spot_market_index
+                    ),
+                    "drift_signer": get_clearing_house_signer_public_key(self.program_id),
+                    "insurance_fund_vault": get_insurance_fund_vault_public_key(
+                        self.program_id, spot_market_index,
+                    ),
+                    "token_program": TOKEN_PROGRAM_ID,
+                }
+            ),
+        ) 
